import pathlib
import json
import sys
import http.client
import re
import os
import time
import hashlib
import ssl
import certifi

context = ssl.create_default_context(cafile=certifi.where())

api = "oa.api2d.net"
cur_dir = pathlib.Path(__file__).parent
posts_dir = cur_dir / "posts"
summary_dir = cur_dir / "summary"
posts = list(posts_dir.glob("**/*.md"))
posts.sort()

summary_force = "force" in sys.argv  # force to summarize, ignore the existing summary
summary_manual = "manual" in sys.argv  # one by one, you can decide whether to summarize
summary_all = "all" in sys.argv  # summarize all automatically
manual_check = "check" in sys.argv  # check the summary results by yourself
summary_counts = 2**32  # summarize counts
for arg in sys.argv:
    if arg.isdigit():
        summary_counts = int(arg)

version = "0.1.0"
forward_key = os.getenv("FORWARD_KEY", "")
if not forward_key:
    print("Please set FORWARD_KEY")
    sys.exit(0)

# keep it longer to avoid misunderstanding with markdown
separator = "################"

model = "gpt-4.1-nano"
# prompt = """我是下面文章的作者，请提取以下文章的主要要点。请用一段简短的话概括以下文章的主旨。可以用1-2句话总结以下文章的重点吗？请使用1-3句话概括以下文章的核心内容。您能从以下文章中提炼出最重要的信息吗？请使用100字以内简要概括。"""
prompt = f"""
以作者本人的视角总结以下文章内容, 注意精简字数, 注意保持语句通畅.

文章是markdown格式, 文章的主要语言是中文, 文章可能包含代码内容.

总结使用一段话. 总结的内容需要准确. 需要通过总结的内容就可以判断文章的大致思想.

不要使用第一人称, 改为"作者"或"本文"等.

注意1: 精简字数, 注意保持语句通畅.
注意2: 以作者本人视角总结, 不要包含你的观点.
注意3: 总结的内容需要准确.

文章整体被{separator}包含.

文章是:
"""

check_prompt = f"""
给你输入一篇文章, 和对这篇文章的总结, 你来判断这个总结是否正确, 是否正确表达了文章的意思.

文章是markdown格式, 文章的主要语言是中文, 文章可能包含代码内容.

用户输入的内容如下表示:

{separator}
<文章>
{separator}

{separator}
<对文章的总结>
{separator}

请判断文章的总结是否正确, 是否正确表达了文章的意思.

你需要输出一个评分, 除了评分之外, 你不需要输出任何内容.

评分细则为:

[A] 完全表达了文章的意思, 总结的内容都在文章中有体现
[B] 表达部分文章的意思, 总结的内容都在文章中有体现
[C] 总结的内容在文章中没有体现, 但是总结的内容和文章的意思相近
[D] 总结的内容和文章内容不以一致, 总结的思想和文章的思想也不一致

注意: 只输出评分, 不要输出其他内容

用户的输入是:
"""

print(f"Summary Prompt: {prompt}")
print(f"Check Prompt: {check_prompt}")

def summary_text(prompt, text, temperature=0):
    connection = http.client.HTTPSConnection(api, context=context)
    payload = json.dumps(
        {
            "model": model,
            "temperature": temperature,
            "messages": [
                {
                    "role": "system",
                    "content": prompt,
                },
                {
                    "role": "user",
                    "content": f"{separator}\n{text}\n{separator}",
                },
            ],
            "safe_mode": False,
        }
    )
    headers = {
        "Authorization": f"Bearer {forward_key}",
        "Content-Type": "application/json",
    }
    connection.request("POST", "/v1/chat/completions", payload, headers)
    response = json.loads(connection.getresponse().read().decode("utf-8"))
    return response["object"] != "error", response

def check_summary(prompt, text, summary):
    connection = http.client.HTTPSConnection(api, context=context)
    payload = json.dumps(
        {
            "model": model,
            "temperature": 0,
            "messages": [
                {
                    "role": "system",
                    "content": prompt,
                },
                {
                    "role": "user",
                    "content": f"{separator}\n{text}\n{separator}\n\n{separator}\n{summary}\n{separator}",
                },
            ],
            "safe_mode": False,
        }
    )
    headers = {
        "Authorization": f"Bearer {forward_key}",
        "Content-Type": "application/json",
    }
    connection.request("POST", "/v1/chat/completions", payload, headers)
    response = json.loads(connection.getresponse().read().decode("utf-8"))
    return response["object"] != "error", response


def get_credit_grants():
    connection = http.client.HTTPSConnection(api, context=context)
    payload = json.dumps(
        {
            "model": "text-davinci-edit-001",
            "instruction": "请修改文本中的拼写错误",
            "input": "What tim is it",
        }
    )
    headers = {
        "Authorization": f"Bearer {forward_key}",
        "Content-Type": "application/json",
    }
    connection.request("GET", "/dashboard/billing/credit_grants", payload, headers)
    return connection.getresponse().read().decode("utf-8")


def iter_posts():
    for post in posts:
        title = post.stem
        mtime = post.stat().st_mtime
        with open(post, "r") as f:
            content = f.read()
            yield title, mtime, content


start_time = time.time()
request_counts = 0
summered_counts = 0
if not summary_dir.exists():
    summary_dir.mkdir()

total_char_counts = 0
for title, mtime, content in iter_posts():
    total_char_counts += len(content)
print(f"Total Characters: {total_char_counts}")

for title, mtime, content in iter_posts():
    char_counts = len(content)
    # short post or draft post or reprint post
    if char_counts < 100 or "draft: true" in content or "reprint: true" in content:
        continue

    slug = re.search(r"slug: \"([^\"]+)\"", content)
    if slug:
        slug = slug.group(1).replace('"', "")
    else:
        slug = title
    print(f"{title} ({slug}): {char_counts}")

    summary_file = summary_dir / (slug + ".json")
    if not summary_force and summary_file.exists():
        with open(summary_file, "r") as f:
            summary_data = json.load(f)

        # workaround for old summary files
        if "md5" not in summary_data:
            summary_data["md5"] = hashlib.md5(content.encode("utf-8")).hexdigest()
            with open(summary_file, "w") as f:
                json.dump(summary_data, f, ensure_ascii=False, indent=2)

        if (
            summary_data["version"] == version
            and summary_data["prompt"] == prompt
            and summary_data["md5"] == hashlib.md5(content.encode("utf-8")).hexdigest()
        ):
            print("Summary Exists")
            continue

    should_summary = not summary_manual or input("Summary? (y/N)") in ["y", "Y"]
    if should_summary:
        summered_counts += 1
        if request_counts == 0:
            start_time = time.time()
        request_counts += 1
        if request_counts > 30 and time.time() - start_time < 60:
            print("Waiting for 60s...")
            time.sleep(60)
            request_counts = 0
            start_time = time.time()

        temperature = 0
        summary_done = False
        check_done = False
        retry_max = 10
        while True:
            retry_max -= 1
            if retry_max <= 0:
                print("Retry Max Exceeded")
                break

            if not summary_done:
                valid, response = summary_text(prompt, content, temperature)
                if not valid:
                    print("Summary Failed!", response)
                    continue
                summary = response["choices"][0]["message"]["content"]
                usage = response["usage"]
                print(f"Summary: {summary}")
            summary_done = True

            if not check_done:
                valid, response = check_summary(check_prompt, content, summary)
                if not valid:
                    print("Check Summary Failed!", response)
                    continue
                score = response["choices"][0]["message"]["content"]
                print(f"Score: {score}")
            check_done = True
            if "A" not in score and "B" not in score and "C" not in score:
                print("Summary Incorrect, Retry...")
                temperature = 0.1
                summary_done, check_done = False, False
                continue

            if not manual_check:
                break
            if input("Summary Correct? (Y/n)") in ["", "y", "Y"]:
                break
            temperature = 0.1
            summary_done, check_done = False, False

        with open(summary_file, "w") as f:
            json.dump(
                {
                    "title": title,
                    "slug": slug,
                    "mtime": mtime,
                    "summary": summary,
                    "usage": usage,
                    "score": score,
                    "prompt": prompt,
                    "version": version,
                    "md5": hashlib.md5(content.encode("utf-8")).hexdigest(),
                    "model": model,
                },
                f,
                ensure_ascii=False,
                indent=2,
            )
        summary_counts -= 1

    # summary all has higher priority
    if summary_counts <= 0 and not summary_all:
        break

if summered_counts > 0:
    print(f"Credits: {get_credit_grants()}")
